using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace ksp_osm
{
    public enum OsmType {
        Node,
        Way,
        Relation
    }
    public struct OsmMember
    {
        public OsmMember(OsmType type,
                              long id,
                              string role = null)
        {
            Type = type;
            Id = id;
            Role = role;
        }

        public OsmType Type { get; }
        public long Id { get; }
        public string Role { get; }

        public static OsmType ParseOsmType(string type)
        {
            switch (type)
            {
                case "node":     return OsmType.Node;
                case "way":      return OsmType.Way;
                case "relation": return OsmType.Relation;
                default:         throw new NotSupportedException();
            }
        }

        public static OsmMember FromXml(XElement xml)
        {
            var role = xml.Attribute("role")?.Value;
            if (role != null) role = string.Intern(role);
            return new OsmMember(
                type: ParseOsmType((string)xml.Attribute("type")),
                id: (long)xml.Attribute("ref"),
                role: role
            );
        }
    }
    public struct Relation
    {
        public Relation(long id,
                        OsmMember[] members,
                        Dictionary<string, string> tags)
        {
            Id = id;
            Members = members;
            Tags = tags;
        }

        public long Id { get; }
        public OsmMember[] Members { get; }
        public Dictionary<string, string> Tags { get; }

        public static Relation FromXml(XElement xml)
        {
            return new Relation(
                id: (long)xml.Attribute("id"),
                members: xml.Elements("member").Select(OsmMember.FromXml).ToArray(),
                tags: Node.ReadTags(xml.Elements("tags"))
            );
        }

        public override string ToString()
        {
            return $"Relation {Id}";
        }
    }
}
